// WideMargin. Simple fast bible software.
// Copyright (C) 2011  Daniel Hughes
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
using System;
using System.Collections.Generic;
using Gtk;
using WideMargin.MVCInterfaces;
using Gdk;
using SharpSword;
using WideMargin.Utilities;

namespace WideMargin.GUI
{
	[System.ComponentModel.ToolboxItem(true)]
	public partial class FilteredView : Gtk.Bin, IFilteredView, IGenericView
	{
		GenericView _genericView;
		IEnumerable<Filter> _filters;
		VBox _filtersBox;
		
		public FilteredView ()
		{
			this.Build ();
			
			HBox hbox = new HBox();
			
			_genericView = new GenericView();
			_genericView.VerseLinkClicked += HandleTextViewVerseLinkClicked;
			_genericView.ActionLinkClicked += HandleTextViewActionLinkClicked;
			_genericView.ScrollbarAtBottom += HandleTextViewScrollbarAtBottom;
			_genericView.RequestVersification += (sender, e) => RequestVersification.Fire(this, e);
			EventBox eventBox = new EventBox();
			eventBox.ModifyBg(StateType.Normal, new Color(255,255,255));
			eventBox.Add(hbox);
			hbox.Add(_genericView);
			
			_filtersBox = new VBox();
			_filtersBox.ModifyBg(StateType.Normal, new Color(255,255,255));
			
			//add filters area
			var scrolledWindow = new ScrolledWindow();
			scrolledWindow.BorderWidth = 0;
			scrolledWindow.HscrollbarPolicy = PolicyType.Never;
			scrolledWindow.ModifyBg(StateType.Normal, new Color(255,255,255));
			Viewport viewport = new Viewport();
			viewport.Add (_filtersBox);
			viewport.ModifyBg(StateType.Normal, new Color(255,255,255));
			viewport.BorderWidth = 0;
			viewport.ShadowType = ShadowType.None;
			scrolledWindow.Add(viewport);
			hbox.PackStart(scrolledWindow, false,false, 0 );
			
			this.Add(eventBox);
			this.ModifyBg(StateType.Normal, new Color(255,255,255));
			
			this.SizeRequested += HandleHandleSizeRequested;
		}
		
		/// <summary>
		/// Occurs when the widget is resized.
		/// It the scrollbars are not visable requests more results
		/// </summary>
		void HandleHandleSizeRequested (object o, SizeRequestedArgs args)
		{
			if(!VScrollbarVisible)
			{
				MoreResultsRequired.Fire(this, EventArgs.Empty);	
			}
		}
		
		public void SetResults(string resultsHtml, IEnumerable<Filter> filters)
		{
			this.Invoke(() =>
			{
				Text = resultsHtml;
				SetFiltersUnsafe(filters);
			});
		}
		
		public event EventHandler<RequestVersificationEventArgs> RequestVersification;
		
		/// <summary>
		/// Occurs when the view requires more search results
		/// </summary>
		public event EventHandler<EventArgs> MoreResultsRequired;
		
		public bool VScrollbarVisible
		{
			get
			{
				return _genericView.VScrollbarVisible;	
			}
		}
		
		protected virtual void HandleTextViewScrollbarAtBottom (object sender, EventArgs e)
		{
			MoreResultsRequired.Fire(sender, EventArgs.Empty);
		}

		void HandleTextViewActionLinkClicked (object sender, LinkClickedEventArgs<string> e)
		{
			ActionLinkClicked.Fire (this, e);
		}

		void HandleTextViewVerseLinkClicked (object sender, LinkClickedEventArgs<IVerseIdentifier> e)
		{
			VerseLinkClicked.Fire (this, e);
		}
		
		public string Text
		{
			set
			{
				_genericView.Text = value;	
			}
		}
		
		public void AppendText(string text)
		{
			this.Invoke(() =>
            {
				_genericView.AppendText(text);
			});
		}
		
		/// <summary>
		/// Gets or sets the filters.
		/// </summary>
		public IEnumerable<Filter> Filters
		{
			set
			{
				this.Invoke(() =>
				{
					SetFiltersUnsafe(value);
				});
			}
			get
			{
				return _filters;	
			}
		}

		protected void SetFiltersUnsafe(IEnumerable<Filter> filters)
		{
			foreach(Widget child in _filtersBox.Children)
			{
				_filtersBox.Remove(child);	
			}
			
			_filters = filters;
			
			foreach(var filter in _filters)
			{
				System.Action view = filter.PerformFilter;
				Button button = new Button(new Label(filter.Name));
				_filtersBox.PackStart(button,false,false,0);
				_filtersBox.BorderWidth = 4;
				button.Clicked += (sender, args) => view();
			}
			ShowAll ();
		}

		public event EventHandler<LinkClickedEventArgs<IVerseIdentifier>> VerseLinkClicked;

		public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;
	}
}

