// WideMargin. Simple fast bible software.
// Copyright (C) 2011  Daniel Hughes
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using WideMargin.MVCInterfaces;
using Gtk;
using WideMargin.Utilities;
using System.Threading;
using SharpSword;
using WebKit;

namespace WideMargin.GUI
{
	/// <summary>
	/// Page which shows a bible passage
	/// </summary>
	[System.ComponentModel.ToolboxItem(true)]
	public partial class PassagePage : Gtk.Bin, IPassageView
	{	
		EventLocker _eventLocker = new EventLocker();
		EventHandler<EventArgs> _requestNext;
		EventHandler<EventArgs> _requestPrevious;
		GenericView _genericView;
		const int _textColumWidth = 450;
		
		/// <summary>
		/// Constructs a PassagePage
		/// </summary>
		public PassagePage ()
		{
			this.Build ();
			
			_genericView = new GenericView();
			_genericView.RequestVersification += Handle_WebViewRequestVersification;
			_genericView.VerseLinkClicked += Handle_WebViewVerseLinkClicked;
			_vbox.Remove(_buttonBox);
			
			_vbox.PackStart(_genericView);
			_vbox.PackEnd(_buttonBox, false, false, 0);
			_previousButton.Image = new Image(ImageCache.Get().LongBackArrow);
			_nextButton.Image = new Image(ImageCache.Get().LongForwardArrow);
			this.ShowAll ();
		}

		void Handle_WebViewVerseLinkClicked (object sender, LinkClickedEventArgs<IVerseIdentifier> e)
		{
			VerseLinkClicked.Fire (this, e);
		}

		void Handle_WebViewRequestVersification (object sender, RequestVersificationEventArgs e)
		{
			RequestVersification.Fire(this, e);
		}
		
		public void AppendText(string text)
		{
			this.Invoke(() =>
           	{
				_genericView.AppendText(text);
			});
		}
		
		/// <summary>
		/// User has clicked on the next button.
		/// Let the controller know.
		/// </summary>
		private void NextButton_Clicked (object sender, System.EventArgs e)
		{
			_requestNext.Fire(sender, new EventArgs());
		}
		
		/// <summary>
		/// User has clicked on the previous button.
		/// Let the controller know.
		/// </summary>
		private void PreviousButton_Clicked (object sender, System.EventArgs e)
		{
			_requestPrevious.Fire(sender, new EventArgs());
		}
		
		/// <summary>
		/// Occurs when the user clicks on the next button.
		/// </summary>
		public event EventHandler<EventArgs> RequestNext
		{
			add
			{	
				_eventLocker.Add(ref _requestNext, value);
			}
			remove
			{
				_eventLocker.Remove(ref _requestNext, value);
			}
		}
		
		/// <summary>
		/// Occurs when the user clicks on the previous button.
		/// </summary>
		public event EventHandler<EventArgs> RequestPrevious
		{
			add
			{	
				_eventLocker.Add(ref _requestPrevious, value);
			}
			remove
			{
				_eventLocker.Remove(ref _requestPrevious, value);
			}
		}
		
		/// <summary>
		/// Show the verses for this passage to the user.
		/// </summary>
		/// <param name="searchResults">
		/// passages to show.
		/// </param>
		public void ShowVerses(IEnumerable<IVerse> searchResults)
		{
			var passageBuilder = new StringBuilder();
			var formatter = SharpSword.Formatters.FormattersManager.Html;
			passageBuilder.Append(formatter.Header());
			passageBuilder.Append(formatter.ChapterTitle(searchResults.First ().Identifier));
			foreach(IVerse verse in searchResults)
			{
				passageBuilder.Append(" ");
				passageBuilder.Append(verse.Text);
			}
			passageBuilder.Append(formatter.Footer());
			
			this.Invoke(() =>
			{
				_genericView.Text = passageBuilder.ToString();
			});
		}
		
		/// <summary>
		/// Scrolls to the specified verse
		/// </summary>
		/// <param name="verse">Verse to scroll to</param>
		public void ScrollToVerse(IVerse verse)
		{
			var script = string.Format("scrollTo('{0}');", verse.Identifier);
			
			this.Invoke(() =>
			{
				_genericView.ExecuteScript(script);
			});
		}
		
		public event EventHandler<LinkClickedEventArgs<IVerseIdentifier>> VerseLinkClicked;

		public event EventHandler<LinkClickedEventArgs<string>> ActionLinkClicked;

		public event EventHandler<RequestVersificationEventArgs> RequestVersification;

		public string Text 
		{
			set 
			{
				throw new NotImplementedException ();
			}
		}
	}
}

